@@ -1,10 +1,11 @@
 import totem
-import gobject, gtk
+import gobject, gtk, gconf
 import gdata.service
 import urllib
 import httplib
 import atom
 import threading
+import time
 import re
 import os
 
@@ -25,7 +26,7 @@ class DownloadThread (threading.Thread):
 
 class YouTube (totem.Plugin):
 	def __init__ (self):
-		totem.Plugin.__init__(self)
+		totem.Plugin.__init__ (self)
 		self.debug = False
 		self.gstreamer_plugins_present = True
 
@@ -50,6 +51,10 @@ class YouTube (totem.Plugin):
 		"""Check for the availability of the flvdemux and souphttpsrc GStreamer plugins"""
 		bvw_name = totem_object.get_video_widget_backend_name ()
 
+		"""If the user's selected 1.5Mbps or greater as their connection speed, grab higher-quality videos
+		   and drop the requirement for the flvdemux plugin."""
+		self.gconf_client = gconf.client_get_default ()
+
 		if bvw_name.find ("GStreamer") != -1:
 			try:
 				import pygst
@@ -57,13 +62,13 @@ class YouTube (totem.Plugin):
 				import gst
 
 				registry = gst.registry_get_default ()
-				if registry.find_plugin ("flvdemux") == None or registry.find_plugin ("soup") == None:
+				if registry.find_plugin ("soup") == None:
 					"""This means an error will be displayed when they try to play anything"""
 					self.gstreamer_plugins_present = False
 			except ImportError:
 				"""Do nothing; either it's using xine or python-gstreamer isn't installed"""
 
-		"""Continue loading the plugin as before"""		
+		"""Continue loading the plugin as before"""
 		self.builder = self.load_interface ("youtube.ui", True, totem_object.get_main_window (), self)
 		self.totem = totem_object
 
@@ -71,6 +76,7 @@ class YouTube (totem.Plugin):
 		self.search_entry.connect ("activate", self.on_search_entry_activated)
 		self.search_button = self.builder.get_object ("yt_search_button")
 		self.search_button.connect ("clicked", self.on_search_button_clicked)
+		self.progress_bar = self.builder.get_object ("yt_progress_bar")
 
 		self.notebook = self.builder.get_object ("yt_notebook")
 		self.notebook.connect ("switch-page", self.on_notebook_page_changed)
@@ -85,7 +91,7 @@ class YouTube (totem.Plugin):
 		totem_object.add_sidebar_page ("youtube", _("YouTube"), self.vbox)
 
 		"""Set up the service"""
-		self.service = gdata.service.GDataService (None, None, "HOSTED_OR_GOOGLE", None, None, "gdata.youtube.com")
+		self.service = gdata.service.GDataService (account_type = "HOSTED_OR_GOOGLE", server = "gdata.youtube.com")
 	def deactivate (self, totem):
 		totem.remove_sidebar_page ("youtube")
 	def setup_treeview (self, treeview_name):
@@ -138,7 +144,13 @@ class YouTube (totem.Plugin):
 		self.youtube_id = youtube_id
 		self.start_index["related"] = 1
 		self.results["related"] = 0
+		self.progress_bar.set_text (_("Fetching related videos..."))
 		self.get_results ("/feeds/api/videos/" + urllib.quote (youtube_id) + "/related?max-results=" + str (self.max_results), "related")
+	def get_fmt_string (self):
+		if self.gconf_client.get_int ("/apps/totem/connection_speed") >= 10:
+			return "&fmt=18"
+		else:
+			return ""
 	def check_url_for_redirects (self, url_path):
 		try:
 			conn = httplib.HTTPConnection ("www.youtube.com")
@@ -163,27 +175,6 @@ class YouTube (totem.Plugin):
 							      self.totem.get_main_window ())
 			return False
 
-		model, rows = treeview.get_selection ().get_selected_rows ()
-		iter = model.get_iter (rows[0])
-		youtube_id = model.get_value (iter, 3)
-
-		"""Get the video stream MRL"""
-		location = self.check_url_for_redirects ("/v/" + urllib.quote (youtube_id))
-
-		if location != False:
-			mrl = "http://www.youtube.com/get_video?video_id=" + urllib.quote (youtube_id) + "&t=" + urllib.quote (re.match (".*[?&]t=([^&]+)", location).groups ()[0])
-			"""location_check = self.check_url_for_redirects (mrl)
-			if location_check != False:
-				mrl = location_check
-			else:
-				mrl = "http://www.youtube.com" + mrl"""
-		else:
-			"""Leave it as taken from the GData feed"""
-			mrl = False
-
-		if mrl != False:
-			model.set_value (iter, 2, mrl)
-
 		return True
 	def on_open_in_web_browser_activated (self, action):
 		model, rows = self.treeview[self.current_treeview_name].get_selection ().get_selected_rows ()
@@ -191,7 +182,7 @@ class YouTube (totem.Plugin):
 		youtube_id = model.get_value (iter, 3)
 
 		"""Open the video in the browser"""
-		os.spawnlp (os.P_NOWAIT, "xdg-open", "xdg-open", "http://www.youtube.com/watch?v=" + urllib.quote (youtube_id))
+		os.spawnlp (os.P_NOWAIT, "xdg-open", "xdg-open", "http://www.youtube.com/watch?v=" + urllib.quote (youtube_id) + self.get_fmt_string ())
 	def on_button_press_event (self, widget, event):
 		self.button_down = True
 	def on_button_release_event (self, widget, event):
@@ -201,6 +192,7 @@ class YouTube (totem.Plugin):
 		"""Load more results when we get near the bottom of the treeview"""
 		if not self.button_down and (adjustment.get_value () + adjustment.page_size) / adjustment.upper > 0.8 and self.results[self.current_treeview_name] >= self.max_results:
 			self.results[self.current_treeview_name] = 0
+			self.progress_bar.set_text (_("Fetching more videos..."))
 			if self.current_treeview_name == "search":
 				self.get_results ("/feeds/api/videos?vq=" + urllib.quote_plus (self.search_terms) + "&max-results=" + str (self.max_results) + "&orderby=relevance&start-index=" + str (self.start_index["search"]), "search", False)
 				if self.debug:
@@ -215,6 +207,9 @@ class YouTube (totem.Plugin):
 	def populate_list_from_results (self, treeview_name):
 		"""Check and acquire the lock"""
 		if self.entry_lock.acquire (False) == False:
+			if (self.last_pulse + 0.035) < time.time():
+				self.progress_bar.pulse ()
+				self.last_pulse = time.time()
 			return True
 
 		"""Return if there are no results (or we've finished)"""
@@ -222,6 +217,8 @@ class YouTube (totem.Plugin):
 			"""Revert the cursor"""
 			window = self.vbox.window
 			window.set_cursor (None)
+			self.progress_bar.set_fraction (0.0)
+			self.progress_bar.set_text ("")
 
 			self.entry[treeview_name] = None
 			self.entry_lock.release ()
@@ -234,6 +231,9 @@ class YouTube (totem.Plugin):
 		self.start_index[treeview_name] += 1
 		youtube_id = self.convert_url_to_id (entry.id.text)
 
+		"""Update the progress bar"""
+		self.progress_bar.set_fraction (float (self.results[treeview_name]) / float (self.max_results))
+
 		self.entry_lock.release ()
 
 		"""Find the content tag"""
@@ -263,6 +263,18 @@ class YouTube (totem.Plugin):
 		"""Don't leak the temporary file"""
 		os.unlink (filename)
 
+		"""Get the video stream MRL"""
+		location = self.check_url_for_redirects ("/v/" + urllib.quote (youtube_id))
+
+		if location != False:
+			mrl = "http://www.youtube.com/get_video?video_id=" + urllib.quote (youtube_id) + "&t=" + urllib.quote (re.match (".*[?&]t=([^&]+)", location).groups ()[0]) + self.get_fmt_string ()
+
+			"""location_check = self.check_url_for_redirects (mrl)
+			if location_check != False:
+				mrl = location_check
+			else:
+				mrl = "http://www.youtube.com" + mrl"""
+
 		self.liststore[treeview_name].append ([pixbuf, entry.title.text, mrl, youtube_id])
 
 		return True
@@ -278,6 +290,7 @@ class YouTube (totem.Plugin):
 		self.search_terms = search_terms
 		self.start_index["search"] = 1
 		self.results["search"] = 0
+		self.progress_bar.set_text (_("Fetching search results..."))
 		self.get_results ("/feeds/api/videos?vq=" + urllib.quote_plus (search_terms) + "&orderby=relevance&max-results=" + str (self.max_results), "search")
 	def on_search_entry_activated (self, entry):
 		self.search_button.clicked ()
@@ -291,6 +304,8 @@ class YouTube (totem.Plugin):
 		"""Give us a nice waiting cursor"""
 		window = self.vbox.window
 		window.set_cursor (gtk.gdk.Cursor (gtk.gdk.WATCH))
+		self.progress_bar.pulse ()
+		self.last_pulse = time.time()
 
 		self.results_downloaded = False
 		DownloadThread (self, url, treeview_name).start ()
